@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtksignal.h>
@@ -31,8 +32,11 @@
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtktable.h>
+#include <gtk/gtkstatusbar.h>
+#include <gtk/gtknotebook.h>
+#include <gtk/gtkvbbox.h>
+#include <gtk/gtkstock.h>
 
-#include "intl.h"
 #include "mgutils.h"
 #include "addressbook.h"
 #include "addressitem.h"
@@ -58,6 +62,7 @@ static struct _AddressEdit_dlg {
 static struct _PersonEdit_dlg {
 	GtkWidget *window;
 	GtkWidget *notebook;
+	GtkWidget *hbbox;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
 	GtkWidget *statusbar;
@@ -146,13 +151,14 @@ static gint edit_address_delete_event(GtkWidget *widget, GdkEventAny *event,
 	return TRUE;
 }
 
-static void edit_address_key_pressed(GtkWidget *widget, GdkEventKey *event,
-				     gboolean *cancelled)
+static gboolean edit_address_key_pressed(GtkWidget *widget, GdkEventKey *event,
+					 gboolean *cancelled)
 {
 	if (event && event->keyval == GDK_Escape) {
 		*cancelled = TRUE;
 		gtk_main_quit();
 	}
+	return FALSE;
 }
 
 static void addressbook_edit_address_create(gboolean *cancelled)
@@ -171,17 +177,17 @@ static void addressbook_edit_address_create(gboolean *cancelled)
 	debug_print("Creating edit_address window...\n");
 
 	window = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_widget_set_usize(window, 400, -1);
+	gtk_widget_set_size_request(window, 400, -1);
 	/* gtk_container_set_border_width(GTK_CONTAINER(window), 8); */
 	gtk_window_set_title(GTK_WINDOW(window), _("Edit address"));
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);	
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(edit_address_delete_event),
-			   cancelled);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(edit_address_key_pressed),
-			   cancelled);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(edit_address_delete_event),
+			 cancelled);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(edit_address_key_pressed),
+			 cancelled);
 
 	vbox = gtk_vbox_new(FALSE, 8);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
@@ -195,15 +201,16 @@ static void addressbook_edit_address_create(gboolean *cancelled)
 	SET_LABEL_AND_ENTRY(_("Address"), addr_entry, 1);
 	SET_LABEL_AND_ENTRY(_("Remarks"), rem_entry,  2);
 
-	gtkut_button_set_create(&hbbox, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
+	gtkut_stock_button_set_create(&hbbox, &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
 
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_address_ok), cancelled);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_address_cancel), cancelled);
+	g_signal_connect(G_OBJECT(ok_btn), "clicked",
+			 G_CALLBACK(edit_address_ok), cancelled);
+	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
+			 G_CALLBACK(edit_address_cancel), cancelled);
 
 	gtk_widget_show_all(vbox);
 
@@ -301,11 +308,12 @@ static gint edit_person_delete_event(GtkWidget *widget, GdkEventAny *event, gboo
 	return TRUE;
 }
 
-static void edit_person_key_pressed(GtkWidget *widget, GdkEventKey *event, gboolean *cancelled) {
+static gboolean edit_person_key_pressed(GtkWidget *widget, GdkEventKey *event, gboolean *cancelled) {
 	if (event && event->keyval == GDK_Escape) {
 		*cancelled = TRUE;
 		gtk_main_quit();
 	}
+	return FALSE;
 }
 
 static gchar *_title_new_ = NULL;
@@ -631,18 +639,18 @@ static void addressbook_edit_person_dialog_create( gboolean *cancelled ) {
 	GtkWidget *hsbox;
 	GtkWidget *statusbar;
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_widget_set_usize(window, EDITPERSON_WIDTH, EDITPERSON_HEIGHT );
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_widget_set_size_request(window, EDITPERSON_WIDTH, EDITPERSON_HEIGHT );
 	/* gtk_container_set_border_width(GTK_CONTAINER(window), 0); */
 	gtk_window_set_title(GTK_WINDOW(window), _("Edit Person Data"));
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);	
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(edit_person_delete_event),
-			   cancelled);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(edit_person_key_pressed),
-			   cancelled);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(edit_person_delete_event),
+			 cancelled);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(edit_person_key_pressed),
+			 cancelled);
 
 	vbox = gtk_vbox_new(FALSE, 4);
 	/* gtk_container_set_border_width(GTK_CONTAINER(vbox), BORDER_WIDTH); */
@@ -662,27 +670,29 @@ static void addressbook_edit_person_dialog_create( gboolean *cancelled ) {
 
 	/* Status line */
 	hsbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, BORDER_WIDTH);
+	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, 0);
 	statusbar = gtk_statusbar_new();
-	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, BORDER_WIDTH);
+	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, 0);
 
 	/* Button panel */
-	gtkut_button_set_create(&hbbox, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
+	gtkut_stock_button_set_create(&hbbox, &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_box_pack_end(GTK_BOX(vnbox), hbbox, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
 
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_person_ok), cancelled);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_person_cancel), cancelled);
-	gtk_signal_connect(GTK_OBJECT(notebook), "switch_page",
-			   GTK_SIGNAL_FUNC(edit_person_switch_page), NULL );
+	g_signal_connect(G_OBJECT(ok_btn), "clicked",
+			 G_CALLBACK(edit_person_ok), cancelled);
+	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
+			 G_CALLBACK(edit_person_cancel), cancelled);
+	g_signal_connect(G_OBJECT(notebook), "switch_page",
+			 G_CALLBACK(edit_person_switch_page), NULL);
 
 	gtk_widget_show_all(vbox);
 
 	personeditdlg.window     = window;
 	personeditdlg.notebook   = notebook;
+	personeditdlg.hbbox      = hbbox;
 	personeditdlg.ok_btn     = ok_btn;
 	personeditdlg.cancel_btn = cancel_btn;
 	personeditdlg.statusbar  = statusbar;
@@ -734,9 +744,9 @@ void addressbook_edit_person_page_basic( gint pageNum, gchar *pageLbl ) {
 	ATTACH_ROW(_("Display Name"), entry_name);
 	locale = conv_get_current_locale();
 	if (locale &&
-	    (!g_strncasecmp(locale, "ja", 2) ||
-	     !g_strncasecmp(locale, "ko", 2) ||
-	     !g_strncasecmp(locale, "zh", 2))) {
+	    (!g_ascii_strncasecmp(locale, "ja", 2) ||
+	     !g_ascii_strncasecmp(locale, "ko", 2) ||
+	     !g_ascii_strncasecmp(locale, "zh", 2))) {
 		ATTACH_ROW(_("Last Name"), entry_ln);
 		ATTACH_ROW(_("First Name"), entry_fn);
 	} else {
@@ -860,7 +870,7 @@ void addressbook_edit_person_page_email( gint pageNum, gchar *pageLbl ) {
 
 	vbuttonbox = gtk_vbutton_box_new();
 	gtk_button_box_set_layout( GTK_BUTTON_BOX(vbuttonbox), GTK_BUTTONBOX_START );
-	gtk_button_box_set_spacing( GTK_BUTTON_BOX(vbuttonbox), 8 );
+	gtk_box_set_spacing( GTK_BOX(vbuttonbox), 8 );
 	gtk_container_set_border_width( GTK_CONTAINER(vbuttonbox), 4 );
 	gtk_container_add( GTK_CONTAINER(vboxb), vbuttonbox );
 
@@ -886,20 +896,20 @@ void addressbook_edit_person_page_email( gint pageNum, gchar *pageLbl ) {
 	gtk_widget_show_all(vbox);
 
 	/* Event handlers */
-	gtk_signal_connect( GTK_OBJECT(clist), "select_row",
-			GTK_SIGNAL_FUNC( edit_person_email_list_selected), NULL );
-	gtk_signal_connect( GTK_OBJECT(buttonUp), "clicked",
-			GTK_SIGNAL_FUNC( edit_person_email_move_up ), NULL );
-	gtk_signal_connect( GTK_OBJECT(buttonDown), "clicked",
-			GTK_SIGNAL_FUNC( edit_person_email_move_down ), NULL );
-	gtk_signal_connect( GTK_OBJECT(buttonDel), "clicked",
-			GTK_SIGNAL_FUNC( edit_person_email_delete ), NULL );
-	gtk_signal_connect( GTK_OBJECT(buttonMod), "clicked",
-			GTK_SIGNAL_FUNC( edit_person_email_modify ), NULL );
-	gtk_signal_connect( GTK_OBJECT(buttonAdd), "clicked",
-			GTK_SIGNAL_FUNC( edit_person_email_add ), NULL );
-	gtk_signal_connect( GTK_OBJECT(buttonClr), "clicked",
-			GTK_SIGNAL_FUNC( edit_person_email_clear ), NULL );
+	g_signal_connect( G_OBJECT(clist), "select_row",
+			  G_CALLBACK( edit_person_email_list_selected), NULL );
+	g_signal_connect( G_OBJECT(buttonUp), "clicked",
+			  G_CALLBACK( edit_person_email_move_up ), NULL );
+	g_signal_connect( G_OBJECT(buttonDown), "clicked",
+			  G_CALLBACK( edit_person_email_move_down ), NULL );
+	g_signal_connect( G_OBJECT(buttonDel), "clicked",
+			  G_CALLBACK( edit_person_email_delete ), NULL );
+	g_signal_connect( G_OBJECT(buttonMod), "clicked",
+			  G_CALLBACK( edit_person_email_modify ), NULL );
+	g_signal_connect( G_OBJECT(buttonAdd), "clicked",
+			  G_CALLBACK( edit_person_email_add ), NULL );
+	g_signal_connect( G_OBJECT(buttonClr), "clicked",
+			  G_CALLBACK( edit_person_email_clear ), NULL );
 
 	personeditdlg.clist_email   = clist;
 	personeditdlg.entry_email   = entry_email;
@@ -999,7 +1009,7 @@ void addressbook_edit_person_page_attrib( gint pageNum, gchar *pageLbl ) {
 
 	vbuttonbox = gtk_vbutton_box_new();
 	gtk_button_box_set_layout( GTK_BUTTON_BOX(vbuttonbox), GTK_BUTTONBOX_START );
-	gtk_button_box_set_spacing( GTK_BUTTON_BOX(vbuttonbox), 8 );
+	gtk_box_set_spacing( GTK_BOX(vbuttonbox), 8 );
 	gtk_container_set_border_width( GTK_CONTAINER(vbuttonbox), 4 );
 	gtk_container_add( GTK_CONTAINER(vboxb), vbuttonbox );
 
@@ -1019,16 +1029,16 @@ void addressbook_edit_person_page_attrib( gint pageNum, gchar *pageLbl ) {
 	gtk_widget_show_all(vbox);
 
 	/* Event handlers */
-	gtk_signal_connect( GTK_OBJECT(clist), "select_row",
-			GTK_SIGNAL_FUNC( edit_person_attrib_list_selected), NULL );
-	gtk_signal_connect( GTK_OBJECT(buttonDel), "clicked",
-			GTK_SIGNAL_FUNC( edit_person_attrib_delete ), NULL );
-	gtk_signal_connect( GTK_OBJECT(buttonMod), "clicked",
-			GTK_SIGNAL_FUNC( edit_person_attrib_modify ), NULL );
-	gtk_signal_connect( GTK_OBJECT(buttonAdd), "clicked",
-			GTK_SIGNAL_FUNC( edit_person_attrib_add ), NULL );
-	gtk_signal_connect( GTK_OBJECT(buttonClr), "clicked",
-			GTK_SIGNAL_FUNC( edit_person_attrib_clear ), NULL );
+	g_signal_connect( G_OBJECT(clist), "select_row",
+			  G_CALLBACK( edit_person_attrib_list_selected), NULL );
+	g_signal_connect( G_OBJECT(buttonDel), "clicked",
+			  G_CALLBACK( edit_person_attrib_delete ), NULL );
+	g_signal_connect( G_OBJECT(buttonMod), "clicked",
+			  G_CALLBACK( edit_person_attrib_modify ), NULL );
+	g_signal_connect( G_OBJECT(buttonAdd), "clicked",
+			  G_CALLBACK( edit_person_attrib_add ), NULL );
+	g_signal_connect( G_OBJECT(buttonClr), "clicked",
+			  G_CALLBACK( edit_person_attrib_clear ), NULL );
 
 	personeditdlg.clist_attrib  = clist;
 	personeditdlg.entry_atname  = entry_name;
@@ -1090,6 +1100,8 @@ ItemPerson *addressbook_edit_person( AddressBookFile *abf, ItemFolder *parent, I
 
 	if (!personeditdlg.window)
 		addressbook_edit_person_create(&cancelled);
+	gtkut_box_set_reverse_order(GTK_BOX(personeditdlg.hbbox),
+				    !prefs_common.comply_gnome_hig);
 	gtk_widget_grab_focus(personeditdlg.ok_btn);
 	gtk_widget_grab_focus(personeditdlg.entry_name);
 	gtk_widget_show(personeditdlg.window);
@@ -1125,10 +1137,10 @@ ItemPerson *addressbook_edit_person( AddressBookFile *abf, ItemFolder *parent, I
 
 	/* Select appropriate start page */
 	if( pgMail ) {
-		gtk_notebook_set_page( GTK_NOTEBOOK(personeditdlg.notebook), PAGE_EMAIL );
+		gtk_notebook_set_current_page( GTK_NOTEBOOK(personeditdlg.notebook), PAGE_EMAIL );
 	}
 	else {
-		gtk_notebook_set_page( GTK_NOTEBOOK(personeditdlg.notebook), PAGE_BASIC );
+		gtk_notebook_set_current_page( GTK_NOTEBOOK(personeditdlg.notebook), PAGE_BASIC );
 	}
 
 	gtk_clist_select_row( GTK_CLIST(personeditdlg.clist_email), 0, 0 );
