@@ -1,7 +1,7 @@
-/* A Bison parser, made by GNU Bison 1.875a.  */
+/* A Bison parser, made by GNU Bison 1.875d.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -253,7 +253,7 @@ int quote_fmtwrap(void)
 
 static int isseparator(int ch)
 {
-	return isspace(ch) || ch == '.' || ch == '-';
+	return g_ascii_isspace(ch) || ch == '.' || ch == '-';
 }
 
 
@@ -292,18 +292,25 @@ typedef union YYSTYPE {
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
+# ifndef YYFREE
+#  define YYFREE free
+# endif
+# ifndef YYMALLOC
+#  define YYMALLOC malloc
+# endif
+
 /* The parser invokes alloca or malloc; define the necessary symbols.  */
 
-# if YYSTACK_USE_ALLOCA
-#  define YYSTACK_ALLOC alloca
+# ifdef YYSTACK_USE_ALLOCA
+#  if YYSTACK_USE_ALLOCA
+#   define YYSTACK_ALLOC alloca
+#  endif
 # else
-#  ifndef YYSTACK_USE_ALLOCA
-#   if defined (alloca) || defined (_ALLOCA_H)
-#    define YYSTACK_ALLOC alloca
-#   else
-#    ifdef __GNUC__
-#     define YYSTACK_ALLOC __builtin_alloca
-#    endif
+#  if defined (alloca) || defined (_ALLOCA_H)
+#   define YYSTACK_ALLOC alloca
+#  else
+#   ifdef __GNUC__
+#    define YYSTACK_ALLOC __builtin_alloca
 #   endif
 #  endif
 # endif
@@ -316,20 +323,20 @@ typedef union YYSTYPE {
 #   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
 #   define YYSIZE_T size_t
 #  endif
-#  define YYSTACK_ALLOC malloc
-#  define YYSTACK_FREE free
+#  define YYSTACK_ALLOC YYMALLOC
+#  define YYSTACK_FREE YYFREE
 # endif
 #endif /* ! defined (yyoverflow) || YYERROR_VERBOSE */
 
 
 #if (! defined (yyoverflow) \
      && (! defined (__cplusplus) \
-	 || (YYSTYPE_IS_TRIVIAL)))
+	 || (defined (YYSTYPE_IS_TRIVIAL) && YYSTYPE_IS_TRIVIAL)))
 
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  short yyss;
+  short int yyss;
   YYSTYPE yyvs;
   };
 
@@ -339,13 +346,13 @@ union yyalloc
 /* The size of an array large to enough to hold all stacks, each with
    N elements.  */
 # define YYSTACK_BYTES(N) \
-     ((N) * (sizeof (short) + sizeof (YYSTYPE))				\
+     ((N) * (sizeof (short int) + sizeof (YYSTYPE))			\
       + YYSTACK_GAP_MAXIMUM)
 
 /* Copy COUNT objects from FROM to TO.  The source and destination do
    not overlap.  */
 # ifndef YYCOPY
-#  if 1 < __GNUC__
+#  if defined (__GNUC__) && 1 < __GNUC__
 #   define YYCOPY(To, From, Count) \
       __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
 #  else
@@ -381,7 +388,7 @@ union yyalloc
 #if defined (__STDC__) || defined (__cplusplus)
    typedef signed char yysigned_char;
 #else
-   typedef short yysigned_char;
+   typedef short int yysigned_char;
 #endif
 
 /* YYFINAL -- State number of the termination state. */
@@ -471,13 +478,13 @@ static const yysigned_char yyrhs[] =
 };
 
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
-static const unsigned short yyrline[] =
+static const unsigned short int yyrline[] =
 {
        0,   154,   154,   157,   158,   161,   162,   166,   170,   174,
      179,   184,   189,   194,   210,   237,   242,   247,   252,   256,
-     261,   266,   288,   312,   336,   362,   366,   370,   374,   378,
-     382,   389,   388,   397,   396,   405,   404,   413,   412,   421,
-     420,   429,   428,   437,   436,   445,   444,   453,   452
+     261,   266,   289,   314,   339,   366,   370,   374,   378,   382,
+     386,   393,   392,   401,   400,   409,   408,   417,   416,   425,
+     424,   433,   432,   441,   440,   449,   448,   457,   456
 };
 #endif
 
@@ -486,18 +493,18 @@ static const unsigned short yyrline[] =
    First, the terminals, then, starting at YYNTOKENS, nonterminals. */
 static const char *const yytname[] =
 {
-  "$end", "error", "$undefined", "SHOW_NEWSGROUPS", "SHOW_DATE", 
-  "SHOW_FROM", "SHOW_FULLNAME", "SHOW_FIRST_NAME", "SHOW_SENDER_INITIAL", 
-  "SHOW_SUBJECT", "SHOW_TO", "SHOW_MESSAGEID", "SHOW_PERCENT", "SHOW_CC", 
-  "SHOW_REFERENCES", "SHOW_MESSAGE", "SHOW_QUOTED_MESSAGE", 
-  "SHOW_BACKSLASH", "SHOW_TAB", "SHOW_QUOTED_MESSAGE_NO_SIGNATURE", 
-  "SHOW_MESSAGE_NO_SIGNATURE", "SHOW_EOL", "SHOW_QUESTION_MARK", 
-  "SHOW_OPARENT", "SHOW_CPARENT", "QUERY_DATE", "QUERY_FROM", 
-  "QUERY_FULLNAME", "QUERY_SUBJECT", "QUERY_TO", "QUERY_NEWSGROUPS", 
-  "QUERY_MESSAGEID", "QUERY_CC", "QUERY_REFERENCES", "OPARENT", "CPARENT", 
-  "CHARACTER", "$accept", "quote_fmt", 
-  "character_or_special_or_query_list", "character_or_special_or_query", 
-  "character", "special", "query", "@1", "@2", "@3", "@4", "@5", "@6", 
+  "$end", "error", "$undefined", "SHOW_NEWSGROUPS", "SHOW_DATE",
+  "SHOW_FROM", "SHOW_FULLNAME", "SHOW_FIRST_NAME", "SHOW_SENDER_INITIAL",
+  "SHOW_SUBJECT", "SHOW_TO", "SHOW_MESSAGEID", "SHOW_PERCENT", "SHOW_CC",
+  "SHOW_REFERENCES", "SHOW_MESSAGE", "SHOW_QUOTED_MESSAGE",
+  "SHOW_BACKSLASH", "SHOW_TAB", "SHOW_QUOTED_MESSAGE_NO_SIGNATURE",
+  "SHOW_MESSAGE_NO_SIGNATURE", "SHOW_EOL", "SHOW_QUESTION_MARK",
+  "SHOW_OPARENT", "SHOW_CPARENT", "QUERY_DATE", "QUERY_FROM",
+  "QUERY_FULLNAME", "QUERY_SUBJECT", "QUERY_TO", "QUERY_NEWSGROUPS",
+  "QUERY_MESSAGEID", "QUERY_CC", "QUERY_REFERENCES", "OPARENT", "CPARENT",
+  "CHARACTER", "$accept", "quote_fmt",
+  "character_or_special_or_query_list", "character_or_special_or_query",
+  "character", "special", "query", "@1", "@2", "@3", "@4", "@5", "@6",
   "@7", "@8", "@9", 0
 };
 #endif
@@ -505,7 +512,7 @@ static const char *const yytname[] =
 # ifdef YYPRINT
 /* YYTOKNUM[YYLEX-NUM] -- Internal token number corresponding to
    token YYLEX-NUM.  */
-static const unsigned short yytoknum[] =
+static const unsigned short int yytoknum[] =
 {
        0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
      265,   266,   267,   268,   269,   270,   271,   272,   273,   274,
@@ -642,7 +649,7 @@ static const unsigned char yystos[] =
 
 #define YYACCEPT	goto yyacceptlab
 #define YYABORT		goto yyabortlab
-#define YYERROR		goto yyerrlab1
+#define YYERROR		goto yyerrorlab
 
 
 /* Like YYERROR except do call yyerror.  This remains here temporarily
@@ -677,11 +684,11 @@ while (0)
    are run).  */
 
 #ifndef YYLLOC_DEFAULT
-# define YYLLOC_DEFAULT(Current, Rhs, N)         \
-  Current.first_line   = Rhs[1].first_line;      \
-  Current.first_column = Rhs[1].first_column;    \
-  Current.last_line    = Rhs[N].last_line;       \
-  Current.last_column  = Rhs[N].last_column;
+# define YYLLOC_DEFAULT(Current, Rhs, N)		\
+   ((Current).first_line   = (Rhs)[1].first_line,	\
+    (Current).first_column = (Rhs)[1].first_column,	\
+    (Current).last_line    = (Rhs)[N].last_line,	\
+    (Current).last_column  = (Rhs)[N].last_column)
 #endif
 
 /* YYLEX -- calling `yylex' with the right arguments.  */
@@ -725,17 +732,17 @@ do {								\
 
 /*------------------------------------------------------------------.
 | yy_stack_print -- Print the state stack from its BOTTOM up to its |
-| TOP (cinluded).                                                   |
+| TOP (included).                                                   |
 `------------------------------------------------------------------*/
 
 #if defined (__STDC__) || defined (__cplusplus)
 static void
-yy_stack_print (short *bottom, short *top)
+yy_stack_print (short int *bottom, short int *top)
 #else
 static void
 yy_stack_print (bottom, top)
-    short *bottom;
-    short *top;
+    short int *bottom;
+    short int *top;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
@@ -765,9 +772,9 @@ yy_reduce_print (yyrule)
 #endif
 {
   int yyi;
-  unsigned int yylineno = yyrline[yyrule];
+  unsigned int yylno = yyrline[yyrule];
   YYFPRINTF (stderr, "Reducing stack by rule %d (line %u), ",
-             yyrule - 1, yylineno);
+             yyrule - 1, yylno);
   /* Print the symbols being reduced, and their result.  */
   for (yyi = yyprhs[yyrule]; 0 <= yyrhs[yyi]; yyi++)
     YYFPRINTF (stderr, "%s ", yytname [yyrhs[yyi]]);
@@ -804,7 +811,7 @@ int yydebug;
    SIZE_MAX < YYSTACK_BYTES (YYMAXDEPTH)
    evaluated with infinite-precision integer arithmetic.  */
 
-#if YYMAXDEPTH == 0
+#if defined (YYMAXDEPTH) && YYMAXDEPTH == 0
 # undef YYMAXDEPTH
 #endif
 
@@ -1002,9 +1009,9 @@ yyparse ()
      to reallocate them elsewhere.  */
 
   /* The state stack.  */
-  short	yyssa[YYINITDEPTH];
-  short *yyss = yyssa;
-  register short *yyssp;
+  short int yyssa[YYINITDEPTH];
+  short int *yyss = yyssa;
+  register short int *yyssp;
 
   /* The semantic value stack.  */
   YYSTYPE yyvsa[YYINITDEPTH];
@@ -1041,6 +1048,7 @@ yyparse ()
   yyssp = yyss;
   yyvsp = yyvs;
 
+
   goto yysetstate;
 
 /*------------------------------------------------------------.
@@ -1066,7 +1074,7 @@ yyparse ()
 	   these so that the &'s don't force the real ones into
 	   memory.  */
 	YYSTYPE *yyvs1 = yyvs;
-	short *yyss1 = yyss;
+	short int *yyss1 = yyss;
 
 
 	/* Each stack pointer address is followed by the size of the
@@ -1094,7 +1102,7 @@ yyparse ()
 	yystacksize = YYMAXDEPTH;
 
       {
-	short *yyss1 = yyss;
+	short int *yyss1 = yyss;
 	union yyalloc *yyptr =
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
@@ -1269,14 +1277,14 @@ yyreduce:
 #line 195 "quote_fmt_parse.y"
     {
 		if (msginfo->fromname) {
-			guchar *p;
+			gchar *p;
 			gchar *str;
 
 			str = alloca(strlen(msginfo->fromname) + 1);
 			if (str != NULL) {
 				strcpy(str, msginfo->fromname);
 				p = str;
-				while (*p && !isspace(*p)) p++;
+				while (*p && !g_ascii_isspace(*p)) p++;
 				*p = '\0';
 				INSERT(str);
 			}
@@ -1290,15 +1298,15 @@ yyreduce:
 #define MAX_SENDER_INITIAL 20
 		if (msginfo->fromname) {
 			gchar tmp[MAX_SENDER_INITIAL];
-			guchar *p;
+			gchar *p;
 			gchar *cur;
 			gint len = 0;
 
 			p = msginfo->fromname;
 			cur = tmp;
 			while (*p) {
-				if (*p && isalnum(*p)) {
-					*cur = toupper(*p);
+				if (*p && g_ascii_isalnum(*p)) {
+					*cur = g_ascii_toupper(*p);
 						cur++;
 					len++;
 					if (len >= MAX_SENDER_INITIAL - 1)
@@ -1371,7 +1379,8 @@ yyreduce:
 			if (body)
 				fp = str_open_as_stream(body);
 			else
-				fp = procmime_get_first_text_content(msginfo);
+				fp = procmime_get_first_text_content
+					(msginfo, NULL);
 
 			if (fp == NULL)
 				g_warning("Can't get text part\n");
@@ -1387,7 +1396,7 @@ yyreduce:
     break;
 
   case 22:
-#line 289 "quote_fmt_parse.y"
+#line 290 "quote_fmt_parse.y"
     {
 		if (msginfo->folder || body) {
 			gchar buf[BUFFSIZE];
@@ -1396,7 +1405,8 @@ yyreduce:
 			if (body)
 				fp = str_open_as_stream(body);
 			else
-				fp = procmime_get_first_text_content(msginfo);
+				fp = procmime_get_first_text_content
+					(msginfo, NULL);
 
 			if (fp == NULL)
 				g_warning("Can't get text part\n");
@@ -1414,7 +1424,7 @@ yyreduce:
     break;
 
   case 23:
-#line 313 "quote_fmt_parse.y"
+#line 315 "quote_fmt_parse.y"
     {
 		if (msginfo->folder || body) {
 			gchar buf[BUFFSIZE];
@@ -1423,7 +1433,8 @@ yyreduce:
 			if (body)
 				fp = str_open_as_stream(body);
 			else
-				fp = procmime_get_first_text_content(msginfo);
+				fp = procmime_get_first_text_content
+					(msginfo, NULL);
 
 			if (fp == NULL)
 				g_warning("Can't get text part\n");
@@ -1441,7 +1452,7 @@ yyreduce:
     break;
 
   case 24:
-#line 337 "quote_fmt_parse.y"
+#line 340 "quote_fmt_parse.y"
     {
 		if (msginfo->folder || body) {
 			gchar buf[BUFFSIZE];
@@ -1450,7 +1461,8 @@ yyreduce:
 			if (body)
 				fp = str_open_as_stream(body);
 			else
-				fp = procmime_get_first_text_content(msginfo);
+				fp = procmime_get_first_text_content
+					(msginfo, NULL);
 
 			if (fp == NULL)
 				g_warning("Can't get text part\n");
@@ -1470,168 +1482,168 @@ yyreduce:
     break;
 
   case 25:
-#line 363 "quote_fmt_parse.y"
+#line 367 "quote_fmt_parse.y"
     {
 		INSERT("\\");
 	}
     break;
 
   case 26:
-#line 367 "quote_fmt_parse.y"
+#line 371 "quote_fmt_parse.y"
     {
 		INSERT("\t");
 	}
     break;
 
   case 27:
-#line 371 "quote_fmt_parse.y"
+#line 375 "quote_fmt_parse.y"
     {
 		INSERT("\n");
 	}
     break;
 
   case 28:
-#line 375 "quote_fmt_parse.y"
+#line 379 "quote_fmt_parse.y"
     {
 		INSERT("?");
 	}
     break;
 
   case 29:
-#line 379 "quote_fmt_parse.y"
+#line 383 "quote_fmt_parse.y"
     {
 		INSERT("{");
 	}
     break;
 
   case 30:
-#line 383 "quote_fmt_parse.y"
+#line 387 "quote_fmt_parse.y"
     {
 		INSERT("}");
 	}
     break;
 
   case 31:
-#line 389 "quote_fmt_parse.y"
+#line 393 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->date != NULL);
 	}
     break;
 
   case 32:
-#line 393 "quote_fmt_parse.y"
+#line 397 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 33:
-#line 397 "quote_fmt_parse.y"
+#line 401 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->from != NULL);
 	}
     break;
 
   case 34:
-#line 401 "quote_fmt_parse.y"
+#line 405 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 35:
-#line 405 "quote_fmt_parse.y"
+#line 409 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->fromname != NULL);
 	}
     break;
 
   case 36:
-#line 409 "quote_fmt_parse.y"
+#line 413 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 37:
-#line 413 "quote_fmt_parse.y"
+#line 417 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->subject != NULL);
 	}
     break;
 
   case 38:
-#line 417 "quote_fmt_parse.y"
+#line 421 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 39:
-#line 421 "quote_fmt_parse.y"
+#line 425 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->to != NULL);
 	}
     break;
 
   case 40:
-#line 425 "quote_fmt_parse.y"
+#line 429 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 41:
-#line 429 "quote_fmt_parse.y"
+#line 433 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->newsgroups != NULL);
 	}
     break;
 
   case 42:
-#line 433 "quote_fmt_parse.y"
+#line 437 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 43:
-#line 437 "quote_fmt_parse.y"
+#line 441 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->msgid != NULL);
 	}
     break;
 
   case 44:
-#line 441 "quote_fmt_parse.y"
+#line 445 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 45:
-#line 445 "quote_fmt_parse.y"
+#line 449 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->cc != NULL);
 	}
     break;
 
   case 46:
-#line 449 "quote_fmt_parse.y"
+#line 453 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 47:
-#line 453 "quote_fmt_parse.y"
+#line 457 "quote_fmt_parse.y"
     {
 		/* add_visibility(msginfo->references != NULL); */
 	}
     break;
 
   case 48:
-#line 457 "quote_fmt_parse.y"
+#line 461 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
@@ -1640,8 +1652,8 @@ yyreduce:
 
     }
 
-/* Line 999 of yacc.c.  */
-#line 1645 "y.tab.c"
+/* Line 1010 of yacc.c.  */
+#line 1657 "y.tab.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1682,18 +1694,33 @@ yyerrlab:
 	{
 	  YYSIZE_T yysize = 0;
 	  int yytype = YYTRANSLATE (yychar);
+	  const char* yyprefix;
 	  char *yymsg;
-	  int yyx, yycount;
+	  int yyx;
 
-	  yycount = 0;
 	  /* Start YYX at -YYN if negative to avoid negative indexes in
 	     YYCHECK.  */
-	  for (yyx = yyn < 0 ? -yyn : 0;
-	       yyx < (int) (sizeof (yytname) / sizeof (char *)); yyx++)
+	  int yyxbegin = yyn < 0 ? -yyn : 0;
+
+	  /* Stay within bounds of both yycheck and yytname.  */
+	  int yychecklim = YYLAST - yyn;
+	  int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
+	  int yycount = 0;
+
+	  yyprefix = ", expecting ";
+	  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
 	    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
-	      yysize += yystrlen (yytname[yyx]) + 15, yycount++;
-	  yysize += yystrlen ("syntax error, unexpected ") + 1;
-	  yysize += yystrlen (yytname[yytype]);
+	      {
+		yysize += yystrlen (yyprefix) + yystrlen (yytname [yyx]);
+		yycount += 1;
+		if (yycount == 5)
+		  {
+		    yysize = 0;
+		    break;
+		  }
+	      }
+	  yysize += (sizeof ("syntax error, unexpected ")
+		     + yystrlen (yytname[yytype]));
 	  yymsg = (char *) YYSTACK_ALLOC (yysize);
 	  if (yymsg != 0)
 	    {
@@ -1702,16 +1729,13 @@ yyerrlab:
 
 	      if (yycount < 5)
 		{
-		  yycount = 0;
-		  for (yyx = yyn < 0 ? -yyn : 0;
-		       yyx < (int) (sizeof (yytname) / sizeof (char *));
-		       yyx++)
+		  yyprefix = ", expecting ";
+		  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
 		    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
 		      {
-			const char *yyq = ! yycount ? ", expecting " : " or ";
-			yyp = yystpcpy (yyp, yyq);
+			yyp = yystpcpy (yyp, yyprefix);
 			yyp = yystpcpy (yyp, yytname[yyx]);
-			yycount++;
+			yyprefix = " or ";
 		      }
 		}
 	      yyerror (yymsg);
@@ -1732,25 +1756,27 @@ yyerrlab:
       /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
-      /* Return failure if at end of input.  */
-      if (yychar == YYEOF)
+      if (yychar <= YYEOF)
         {
-	  /* Pop the error token.  */
-          YYPOPSTACK;
-	  /* Pop the rest of the stack.  */
-	  while (yyss < yyssp)
-	    {
-	      YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
-	      yydestruct (yystos[*yyssp], yyvsp);
-	      YYPOPSTACK;
-	    }
-	  YYABORT;
+          /* If at end of input, pop the error token,
+	     then the rest of the stack, then return failure.  */
+	  if (yychar == YYEOF)
+	     for (;;)
+	       {
+		 YYPOPSTACK;
+		 if (yyssp == yyss)
+		   YYABORT;
+		 YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
+		 yydestruct (yystos[*yyssp], yyvsp);
+	       }
         }
+      else
+	{
+	  YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
+	  yydestruct (yytoken, &yylval);
+	  yychar = YYEMPTY;
 
-      YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
-      yydestruct (yytoken, &yylval);
-      yychar = YYEMPTY;
-
+	}
     }
 
   /* Else will try to reuse lookahead token after shifting the error
@@ -1758,9 +1784,27 @@ yyerrlab:
   goto yyerrlab1;
 
 
-/*----------------------------------------------------.
-| yyerrlab1 -- error raised explicitly by an action.  |
-`----------------------------------------------------*/
+/*---------------------------------------------------.
+| yyerrorlab -- error raised explicitly by YYERROR.  |
+`---------------------------------------------------*/
+yyerrorlab:
+
+#ifdef __GNUC__
+  /* Pacify GCC when the user code never invokes YYERROR and the label
+     yyerrorlab therefore never appears in user code.  */
+  if (0)
+     goto yyerrorlab;
+#endif
+
+  yyvsp -= yylen;
+  yyssp -= yylen;
+  yystate = *yyssp;
+  goto yyerrlab1;
+
+
+/*-------------------------------------------------------------.
+| yyerrlab1 -- common code for both syntax error and YYERROR.  |
+`-------------------------------------------------------------*/
 yyerrlab1:
   yyerrstatus = 3;	/* Each real token shifted decrements this.  */
 
@@ -1784,9 +1828,8 @@ yyerrlab1:
 
       YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
       yydestruct (yystos[yystate], yyvsp);
-      yyvsp--;
-      yystate = *--yyssp;
-
+      YYPOPSTACK;
+      yystate = *yyssp;
       YY_STACK_PRINT (yyss, yyssp);
     }
 
