@@ -35,6 +35,7 @@
 
 #include <gpgme.h>
 
+#include "procmsg.h"
 #include "procmime.h"
 #include "procheader.h"
 #include "base64.h"
@@ -50,6 +51,8 @@
 
 #define DIM(v)     (sizeof(v) / sizeof((v)[0]))
 
+static gboolean gpg_available = TRUE;
+
 static gchar *content_names[] = {
 	"Content-Type",
 	"Content-Disposition",
@@ -105,8 +108,12 @@ static void dump_part(MimeInfo *mimeinfo, FILE *fp)
 
 void rfc2015_disable_all(void)
 {
-	/* FIXME: set a flag, so that we don't bother the user with failed
-	 * gpgme messages */
+	gpg_available = FALSE;
+}
+
+gboolean rfc2015_is_available(void)
+{
+	return gpg_available;
 }
 
 void rfc2015_secure_remove(const gchar *fname)
@@ -165,7 +172,10 @@ static gchar *sig_status_full(gpgme_ctx_t ctx, gpgme_verify_result_t result)
 			ctime_val = localtime(&created);
 			my_strftime(ctime_str, sizeof (ctime_str), "%c",
 			            ctime_val);
-			ctime_str_utf8 = conv_localetodisp(ctime_str, NULL);
+			ctime_str_utf8 = g_locale_to_utf8(ctime_str, -1,
+							  NULL, NULL, NULL);
+			if (!ctime_str_utf8)
+				ctime_str_utf8 = g_strdup(ctime_str);
 			g_string_sprintfa(str, _("Signature made at %s\n"),
 			                  ctime_str_utf8);
 			g_free(ctime_str_utf8);
@@ -201,6 +211,11 @@ static void check_signature(MimeInfo *mimeinfo, MimeInfo *partinfo, FILE *fp)
 		goto leave;
 	}
 
+	if (rfc2015_is_pkcs7_signature_part(partinfo)) {
+		debug_print("pkcs7 signature detected\n");
+		gpgme_set_protocol(ctx, GPGME_PROTOCOL_CMS);
+	}
+
 	/* don't include the last empty line.
 	   It does not belong to the signed text */
 	if (mimeinfo->children->size > 0) {
@@ -252,6 +267,17 @@ static void check_signature(MimeInfo *mimeinfo, MimeInfo *partinfo, FILE *fp)
 		goto leave;
 	}
 
+#if 0
+	if (partinfo->encoding_type == ENC_BASE64) {
+		err = gpgme_data_set_encoding(sig, GPGME_DATA_ENCODING_BASE64);
+		if (err) {
+			debug_print("gpgme_data_set_encoding failed: %s\n",
+			            gpgme_strerror (err));
+			goto leave;
+		}
+	}
+#endif
+
 	err = gpgme_op_verify(ctx, sig, text, NULL);
 	if (err)  {
 		debug_print("gpgme_op_verify failed: %s\n",
@@ -411,8 +437,7 @@ MimeInfo **rfc2015_find_signature(MimeInfo *mimeinfo)
 	/* check that we have at least 2 parts of the correct type */
 	for (partinfo = mimeinfo->children;
 	        partinfo != NULL; partinfo = partinfo->next) {
-		if (++n > 1  && !g_ascii_strcasecmp(partinfo->content_type,
-		                                    "application/pgp-signature"))
+		if (++n > 1  && rfc2015_is_signature_part(partinfo))
 			break;
 	}
 
@@ -450,6 +475,34 @@ void rfc2015_check_signature(MimeInfo *mimeinfo, FILE *fp)
 	g_free(signedinfo);
 }
 
+gboolean rfc2015_is_pgp_signature_part(MimeInfo *mimeinfo)
+{
+	if (!mimeinfo || !mimeinfo->content_type)
+		return FALSE;
+
+	return !g_ascii_strcasecmp(mimeinfo->content_type,
+				   "application/pgp-signature");
+}
+
+gboolean rfc2015_is_pkcs7_signature_part(MimeInfo *mimeinfo)
+{
+	const gchar *c_type;
+
+	if (!mimeinfo || !mimeinfo->content_type)
+		return FALSE;
+
+	c_type = mimeinfo->content_type;
+
+	return (!g_ascii_strcasecmp(c_type, "application/pkcs7-signature") ||
+		!g_ascii_strcasecmp(c_type, "application/x-pkcs7-signature"));
+}
+
+gboolean rfc2015_is_signature_part(MimeInfo *mimeinfo)
+{
+	return (rfc2015_is_pgp_signature_part(mimeinfo) ||
+		rfc2015_is_pkcs7_signature_part(mimeinfo));
+}
+
 gint rfc2015_is_encrypted(MimeInfo *mimeinfo)
 {
 	if (!mimeinfo || mimeinfo->mime_type != MIME_MULTIPART)
@@ -637,6 +690,55 @@ void rfc2015_decrypt_message(MsgInfo *msginfo, MimeInfo *mimeinfo, FILE *fp)
 
 #undef DECRYPTION_ABORT
 
+FILE *rfc2015_open_message_decrypted(MsgInfo *msginfo, MimeInfo **mimeinfo)
+{
+	FILE *fp;
+	MimeInfo *mimeinfo_;
+	glong fpos;
+
+	g_return_val_if_fail(msginfo != NULL, NULL);
+
+	if (mimeinfo) *mimeinfo = NULL;
+
+	if ((fp = procmsg_open_message(msginfo)) == NULL) return NULL;
+
+	mimeinfo_ = procmime_scan_mime_header(fp);
+	if (!mimeinfo_) {
+		fclose(fp);
+		return NULL;
+	}
+
+	if (!MSG_IS_ENCRYPTED(msginfo->flags) &&
+	    rfc2015_is_encrypted(mimeinfo_)) {
+		MSG_SET_TMP_FLAGS(msginfo->flags, MSG_ENCRYPTED);
+	}
+
+	if (MSG_IS_ENCRYPTED(msginfo->flags) &&
+	    !msginfo->plaintext_file &&
+	    !msginfo->decryption_failed) {
+		fpos = ftell(fp);
+		rfc2015_decrypt_message(msginfo, mimeinfo_, fp);
+		if (msginfo->plaintext_file &&
+		    !msginfo->decryption_failed) {
+			fclose(fp);
+			procmime_mimeinfo_free_all(mimeinfo_);
+			if ((fp = procmsg_open_message(msginfo)) == NULL)
+				return NULL;
+			mimeinfo_ = procmime_scan_mime_header(fp);
+			if (!mimeinfo_) {
+				fclose(fp);
+				return NULL;
+			}
+		} else {
+			if (fseek(fp, fpos, SEEK_SET) < 0)
+				perror("fseek");
+		}
+	}
+
+	if (mimeinfo) *mimeinfo = mimeinfo_;
+	return fp;
+}
+
 
 /*
  * plain contains an entire mime object.
