@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -203,6 +203,9 @@ static void textview_popup_menu_activate_image_cb
 						(GtkMenuItem	*menuitem,
 						 gpointer	 data);
 
+static void textview_adj_value_changed		(GtkAdjustment	*adj,
+						 gpointer	 data);
+
 static void textview_smooth_scroll_do		(TextView	*textview,
 						 gfloat		 old_value,
 						 gfloat		 last_value,
@@ -249,7 +252,7 @@ TextView *textview_create(void)
 				       GTK_POLICY_AUTOMATIC,
 				       GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwin),
-					    GTK_SHADOW_IN);
+					    GTK_SHADOW_ETCHED_IN);
 	gtk_widget_set_size_request
 		(scrolledwin, prefs_common.mainview_width, -1);
 
@@ -282,6 +285,10 @@ TextView *textview_create(void)
 	g_signal_connect(G_OBJECT(text), "populate-popup",
 			 G_CALLBACK(textview_populate_popup), textview);
 
+	g_signal_connect(G_OBJECT(GTK_TEXT_VIEW(text)->vadjustment),
+			 "value-changed",
+			 G_CALLBACK(textview_adj_value_changed), textview);
+
 	gtk_widget_show(scrolledwin);
 
 	vbox = gtk_vbox_new(FALSE, 0);
@@ -311,7 +318,7 @@ static void textview_create_tags(TextView *textview)
 	buffer = gtk_text_view_get_buffer(text);
 
 	gtk_text_buffer_create_tag(buffer, "header",
-				   "pixels-above-lines", 0,
+				   "pixels-above-lines", 1,
 				   "pixels-above-lines-set", TRUE,
 				   "pixels-below-lines", 0,
 				   "pixels-below-lines-set", TRUE,
@@ -321,6 +328,14 @@ static void textview_create_tags(TextView *textview)
 				   "weight", PANGO_WEIGHT_BOLD,
 				   NULL);
 
+	gtk_text_buffer_create_tag(buffer, "mimepart",
+				   "pixels-above-lines", 1,
+				   "pixels-above-lines-set", TRUE,
+				   "pixels-below-lines", 1,
+				   "pixels-below-lines-set", TRUE,
+				   "font-desc", font_desc,
+				   NULL);
+
 	textview->quote0_tag =
 		gtk_text_buffer_create_tag(buffer, "quote0",
 					   "foreground-gdk", &quote_colors[0],
@@ -441,6 +456,8 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 		charset = prefs_common.force_charset;
 	else if (mimeinfo->charset)
 		charset = mimeinfo->charset;
+	else if (prefs_common.default_encoding)
+		charset = prefs_common.default_encoding;
 
 	textview_set_font(textview, charset);
 	textview_clear(textview);
@@ -472,6 +489,10 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	const gchar *charset = NULL;
 	GPtrArray *headers = NULL;
 	gboolean is_rfc822_part = FALSE;
+	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
+	GtkTextMark *mark;
 
 	g_return_if_fail(mimeinfo != NULL);
 	g_return_if_fail(fp != NULL);
@@ -493,6 +514,8 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		charset = prefs_common.force_charset;
 	else if (mimeinfo->charset)
 		charset = mimeinfo->charset;
+	else if (prefs_common.default_encoding)
+		charset = prefs_common.default_encoding;
 
 	if (!boundary && mimeinfo->mime_type == MIME_TEXT) {
 		if (fseek(fp, mimeinfo->fpos, SEEK_SET) < 0)
@@ -540,18 +563,14 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	}
 
 	textview_set_font(textview, charset);
-
 	textview_clear(textview);
 
-	if (headers) {
-		GtkTextView *text = GTK_TEXT_VIEW(textview->text);
-		GtkTextBuffer *buffer;
-		GtkTextIter iter;
+	buffer = gtk_text_view_get_buffer(text);
 
+	if (headers) {
 		textview_show_header(textview, headers);
 		procheader_header_array_destroy(headers);
 
-		buffer = gtk_text_view_get_buffer(text);
 		gtk_text_buffer_get_end_iter(buffer, &iter);
 		textview->body_pos = gtk_text_iter_get_offset(&iter);
 		if (!mimeinfo->main)
@@ -562,6 +581,10 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		textview_add_parts(textview, mimeinfo, fp);
 	else
 		textview_write_body(textview, mimeinfo, fp, charset);
+
+	textview_set_position(textview, 0);
+	mark = gtk_text_buffer_get_insert(buffer);
+	gtk_text_view_scroll_mark_onscreen(text, mark);
 }
 
 static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
@@ -602,6 +625,8 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		charset = prefs_common.force_charset;
 	else if (mimeinfo->charset)
 		charset = mimeinfo->charset;
+	else if (prefs_common.default_encoding)
+		charset = prefs_common.default_encoding;
 
 	if (mimeinfo->mime_type == MIME_MESSAGE_RFC822) {
 		headers = textview_scan_header(textview, fp, charset);
@@ -642,12 +667,14 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		else
 			color = "nocheck-signature";
 		gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, buf, -1,
-							 color, NULL);
+							 color, "mimepart",
+							 NULL);
 	} else
 #endif
 	if (mimeinfo->mime_type != MIME_TEXT &&
 	    mimeinfo->mime_type != MIME_TEXT_HTML) {
-		gtk_text_buffer_insert(buffer, &iter, buf, -1);
+		gtk_text_buffer_insert_with_tags_by_name
+			(buffer, &iter, buf, -1, "mimepart", NULL);
 		if (mimeinfo->mime_type == MIME_IMAGE &&
 		    prefs_common.inline_image) {
 			GdkPixbuf *pixbuf;
@@ -704,7 +731,8 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		if (!mimeinfo->main &&
 		    mimeinfo->parent &&
 		    mimeinfo->parent->children != mimeinfo)
-			gtk_text_buffer_insert(buffer, &iter, buf, -1);
+			gtk_text_buffer_insert_with_tags_by_name
+				(buffer, &iter, buf, -1, "mimepart", NULL);
 		else
 			gtk_text_buffer_insert(buffer, &iter, "\n", 1);
 		textview_write_body(textview, mimeinfo, fp, charset);
@@ -806,7 +834,7 @@ static gboolean get_uri_part(const gchar *start, const gchar *scanpos,
 
 	/* find end point of URI */
 	for (ep_ = scanpos; *ep_ != '\0'; ep_++) {
-		if (!isgraph(*(const guchar *)ep_) ||
+		if (!g_ascii_isgraph(*ep_) ||
 		    !isascii(*(const guchar *)ep_) ||
 		    strchr("()<>{}[]\"", *ep_))
 			break;
@@ -818,11 +846,9 @@ static gboolean get_uri_part(const gchar *start, const gchar *scanpos,
 	 * should pass some URI type to this function and decide on that whether
 	 * to perform punctuation stripping */
 
-#define IS_REAL_PUNCT(ch)	(ispunct(ch) && ((ch) != '/')) 
+#define IS_REAL_PUNCT(ch)	(g_ascii_ispunct(ch) && !strchr("/?=", ch)) 
 
-	for (; ep_ - 1 > scanpos + 1 &&
-	       IS_REAL_PUNCT(*(const guchar *)(ep_ - 1));
-	     ep_--)
+	for (; ep_ - 1 > scanpos + 1 && IS_REAL_PUNCT(*(ep_ - 1)); ep_--)
 		;
 
 #undef IS_REAL_PUNCT
@@ -837,6 +863,18 @@ static gchar *make_uri_string(const gchar *bp, const gchar *ep)
 	return g_strndup(bp, ep - bp);
 }
 
+static gchar *make_http_uri_string(const gchar *bp, const gchar *ep)
+{
+	gchar *tmp;
+	gchar *result;
+
+	tmp = g_strndup(bp, ep - bp);
+	result = g_strconcat("http://", tmp, NULL);
+	g_free(tmp);
+
+	return result;
+}
+
 /* valid mail address characters */
 #define IS_RFC822_CHAR(ch) \
 	(isascii(ch) && \
@@ -955,7 +993,7 @@ static void textview_make_clickable_parts(TextView *textview,
 		{"http://",  strcasestr, get_uri_part,   make_uri_string},
 		{"https://", strcasestr, get_uri_part,   make_uri_string},
 		{"ftp://",   strcasestr, get_uri_part,   make_uri_string},
-		{"www.",     strcasestr, get_uri_part,   make_uri_string},
+		{"www.",     strcasestr, get_uri_part,   make_http_uri_string},
 		{"mailto:",  strcasestr, get_uri_part,   make_uri_string},
 		{"@",        strcasestr, get_email_part, make_email_string}
 	};
@@ -1218,58 +1256,20 @@ void textview_set_position(TextView *textview, gint pos)
 static GPtrArray *textview_scan_header(TextView *textview, FILE *fp,
 				       const gchar *encoding)
 {
-	gchar buf[BUFFSIZE];
-	GPtrArray *headers, *sorted_headers;
-	GSList *disphdr_list;
-	Header *header;
-	gint i;
-
 	g_return_val_if_fail(fp != NULL, NULL);
 
 	if (textview->show_all_headers)
 		return procheader_get_header_array_asis(fp, encoding);
 
 	if (!prefs_common.display_header) {
+		gchar buf[BUFFSIZE];
+
 		while (fgets(buf, sizeof(buf), fp) != NULL)
 			if (buf[0] == '\r' || buf[0] == '\n') break;
 		return NULL;
 	}
 
-	headers = procheader_get_header_array_asis(fp, encoding);
-
-	sorted_headers = g_ptr_array_new();
-
-	for (disphdr_list = prefs_common.disphdr_list; disphdr_list != NULL;
-	     disphdr_list = disphdr_list->next) {
-		DisplayHeaderProp *dp =
-			(DisplayHeaderProp *)disphdr_list->data;
-
-		for (i = 0; i < headers->len; i++) {
-			header = g_ptr_array_index(headers, i);
-
-			if (!g_ascii_strcasecmp(header->name, dp->name)) {
-				if (dp->hidden)
-					procheader_header_free(header);
-				else
-					g_ptr_array_add(sorted_headers, header);
-
-				g_ptr_array_remove_index(headers, i);
-				i--;
-			}
-		}
-	}
-
-	if (prefs_common.show_other_header) {
-		for (i = 0; i < headers->len; i++) {
-			header = g_ptr_array_index(headers, i);
-			g_ptr_array_add(sorted_headers, header);
-		}
-		g_ptr_array_free(headers, TRUE);
-	} else
-		procheader_header_array_destroy(headers);
-
-
-	return sorted_headers;
+	return procheader_get_header_array_for_display(fp, encoding);
 }
 
 static void textview_show_header(TextView *textview, GPtrArray *headers)
@@ -1283,12 +1283,12 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 	g_return_if_fail(headers != NULL);
 
 	buffer = gtk_text_view_get_buffer(text);
+	gtk_text_buffer_get_end_iter(buffer, &iter);
 
 	for (i = 0; i < headers->len; i++) {
 		header = g_ptr_array_index(headers, i);
 		g_return_if_fail(header->name != NULL);
 
-		gtk_text_buffer_get_end_iter(buffer, &iter);
 		gtk_text_buffer_insert_with_tags_by_name
 			(buffer, &iter, header->name, -1,
 			 "header_title", "header", NULL);
@@ -1306,7 +1306,6 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 		    (!strncmp(header->name, "X-Mailer", 8) ||
 		     !strncmp(header->name, "X-Newsreader", 12)) &&
 		    strstr(header->body, "Sylpheed") != NULL) {
-			gtk_text_buffer_get_end_iter(buffer, &iter);
 			gtk_text_buffer_insert_with_tags_by_name
 				(buffer, &iter, header->body, -1,
 				 "header", "emphasis", NULL);
@@ -1317,7 +1316,7 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 			textview_make_clickable_parts
 				(textview, "header", NULL, header->body);
 		}
-		gtk_text_buffer_get_end_iter(buffer, &iter); //
+		gtk_text_buffer_get_end_iter(buffer, &iter);
 		gtk_text_buffer_insert_with_tags_by_name
 			(buffer, &iter, "\n", 1, "header", NULL);
 	}
@@ -1474,11 +1473,17 @@ static void textview_smooth_scroll_do(TextView *textview,
 
 	/* gdk_key_repeat_disable(); */
 
+	g_signal_handlers_block_by_func(vadj, textview_adj_value_changed,
+					textview);
+
 	for (i = step; i <= change_value; i += step) {
 		vadj->value = old_value + (up ? -i : i);
 		g_signal_emit_by_name(G_OBJECT(vadj), "value_changed", 0);
 	}
 
+	g_signal_handlers_unblock_by_func(vadj, textview_adj_value_changed,
+					  textview);
+
 	vadj->value = last_value;
 	g_signal_emit_by_name(G_OBJECT(vadj), "value_changed", 0);
 
@@ -1586,8 +1591,18 @@ static gboolean textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 	case GDK_End:
 	case GDK_Control_L:
 	case GDK_Control_R:
+	case GDK_KP_Tab:
+	case GDK_KP_Home:
+	case GDK_KP_Left:
+	case GDK_KP_Up:
+	case GDK_KP_Right:
+	case GDK_KP_Down:
+	case GDK_KP_Page_Up:
+	case GDK_KP_Page_Down:
+	case GDK_KP_End:
 		break;
 	case GDK_space:
+	case GDK_KP_Space:
 		if (summaryview)
 			summary_pass_key_press_event(summaryview, event);
 		else
@@ -1600,11 +1615,13 @@ static gboolean textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 		textview_scroll_page(textview, TRUE);
 		break;
 	case GDK_Return:
+	case GDK_KP_Enter:
 		textview_scroll_one_line
 			(textview, (event->state &
 				    (GDK_SHIFT_MASK|GDK_MOD1_MASK)) != 0);
 		break;
 	case GDK_Delete:
+	case GDK_KP_Delete:
 		if (summaryview)
 			summary_pass_key_press_event(summaryview, event);
 		break;
@@ -1761,11 +1778,7 @@ static void textview_show_uri(TextView *textview, GtkTextIter *start,
 	STATUSBAR_POP(textview);
 	uri = textview_get_uri(textview, start, end);
 	if (uri) {
-		gchar *trimmed_uri;
-
-		trimmed_uri = trim_string(uri->uri, 60);
-		STATUSBAR_PUSH(textview, trimmed_uri);
-		g_free(trimmed_uri);
+		STATUSBAR_PUSH(textview, uri->uri);
 	}
 }
 
@@ -2034,6 +2047,17 @@ static void textview_popup_menu_activate_image_cb(GtkMenuItem *menuitem,
 	g_free(src);
 }
 
+static void textview_adj_value_changed(GtkAdjustment *adj, gpointer data)
+{
+	TextView *textview = (TextView *)data;
+	GtkTextBuffer *buffer;
+
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview->text));
+	if (gtk_text_buffer_get_selection_bounds(buffer, NULL, NULL))
+		return;
+	gtk_text_view_place_cursor_onscreen(GTK_TEXT_VIEW(textview->text));
+}
+
 static gboolean textview_uri_security_check(TextView *textview, RemoteURI *uri)
 {
 	GtkTextBuffer *buffer;
@@ -2058,7 +2082,7 @@ static gboolean textview_uri_security_check(TextView *textview, RemoteURI *uri)
 
 		uri_path = get_uri_path(uri->uri);
 		visible_uri_path = get_uri_path(visible_str);
-		if (strcmp(uri_path, visible_uri_path) != 0)
+		if (path_cmp(uri_path, visible_uri_path) != 0)
 			retval = FALSE;
 	}
 
